@@ -14,7 +14,7 @@ import numpy as np
 
 from .nestedsamplers import _SAMPLING, SuperSampler
 from .dynamicsampler import (DynamicSampler, _get_update_interval_ratio,
-                             _SAMPLERS, initialize_live_points)
+                             _SAMPLERS, _initialize_live_points)
 from .utils import (LogLikelihood, get_random_generator, get_enlarge_bootstrap,
                     get_nonbounded)
 
@@ -47,7 +47,7 @@ def _get_citations(nested_type, bound, sampler):
     # Main references for the code
     default_refs = [
         ("Speagle (2020)", "ui.adsabs.harvard.edu/abs/2020MNRAS.493.3132S"),
-        ("Koposov et al. (2022)", "doi.org/10.5281/zenodo.3348367")
+        ("Koposov et al. (2023)", "doi.org/10.5281/zenodo.3348367")
     ]
 
     # Basics of nested sampling algorithm
@@ -216,52 +216,12 @@ def _parse_pool_queue(pool, queue_size):
     return M, queue_size
 
 
-class NestedSampler(SuperSampler):
+def _assemble_sampler_docstring(dynamic):
     """
-    The main class performing the static nested sampling.
-    It inherits all the methods of the dynesty.sampler.SuperSampler.
+    Assemble the docstring for the NestedSampler and DynamicNestedSampler
+    We do that to avoid duplicating the parameter descriptions
     """
-
-    def __new__(cls,
-                loglikelihood,
-                prior_transform,
-                ndim,
-                nlive=500,
-                bound='multi',
-                sample='auto',
-                periodic=None,
-                reflective=None,
-                update_interval=None,
-                first_update=None,
-                npdim=None,
-                rstate=None,
-                queue_size=None,
-                pool=None,
-                use_pool=None,
-                live_points=None,
-                logl_args=None,
-                logl_kwargs=None,
-                ptform_args=None,
-                ptform_kwargs=None,
-                gradient=None,
-                grad_args=None,
-                grad_kwargs=None,
-                compute_jac=False,
-                enlarge=None,
-                bootstrap=None,
-                walks=None,
-                facc=0.5,
-                slices=None,
-                fmove=0.9,
-                max_move=100,
-                update_func=None,
-                ncdim=None,
-                blob=False,
-                save_history=False,
-                history_filename=None):
-        """
-        Initializes and returns a sampler object for Static Nested Sampling.
-
+    common = """
         Parameters
         ----------
         loglikelihood : function
@@ -500,19 +460,82 @@ optional
             just sample uniformly from the prior distribution.
             If this is `None` (default), this will default to npdim.
 
-        blob: bool, optional 
+        blob: bool, optional
             The default value is False. If it is true, then the log-likelihood
-            should return the tuple of logl and a numpy-array "blob" that will 
-            stored as part of the chain. That blob can contain auxiliary 
+            should return the tuple of logl and a numpy-array "blob" that will
+            stored as part of the chain. That blob can contain auxiliary
             information computed inside the likelihood function.
+    """
 
+    static_docstring = f"""
+        Initializes and returns a sampler object for Static Nested Sampling.
+{common}
         Returns
         -------
         sampler : sampler from :mod:`~dynesty.nestedsamplers`
             An initialized instance of the chosen sampler specified via
             `bound`.
+        """
+
+    dynamic_docstring = f"""
+        Initializes a sampler object for Dynamic Nested Sampling.
+
+{common}
+        Returns
+        -------
+        sampler : a :class:`dynesty.DynamicSampler` instance
+            An initialized instance of the dynamic nested sampler.
 
         """
+    if dynamic:
+        return dynamic_docstring
+    else:
+        return static_docstring
+
+
+class NestedSampler(SuperSampler):
+    """
+    The main class performing the static nested sampling.
+    It inherits all the methods of the dynesty.sampler.SuperSampler.
+    """
+
+    def __new__(cls,
+                loglikelihood,
+                prior_transform,
+                ndim,
+                nlive=500,
+                bound='multi',
+                sample='auto',
+                periodic=None,
+                reflective=None,
+                update_interval=None,
+                first_update=None,
+                npdim=None,
+                rstate=None,
+                queue_size=None,
+                pool=None,
+                use_pool=None,
+                live_points=None,
+                logl_args=None,
+                logl_kwargs=None,
+                ptform_args=None,
+                ptform_kwargs=None,
+                gradient=None,
+                grad_args=None,
+                grad_kwargs=None,
+                compute_jac=False,
+                enlarge=None,
+                bootstrap=None,
+                walks=None,
+                facc=0.5,
+                slices=None,
+                fmove=0.9,
+                max_move=100,
+                update_func=None,
+                ncdim=None,
+                blob=False,
+                save_history=False,
+                history_filename=None):
 
         # Prior dimensions.
         if npdim is None:
@@ -639,16 +662,16 @@ optional
             kwargs['grad'] = grad
             kwargs['compute_jac'] = compute_jac
 
-        live_points = initialize_live_points(live_points,
-                                             ptform,
-                                             loglike,
-                                             M,
-                                             nlive=nlive,
-                                             npdim=npdim,
-                                             rstate=rstate,
-                                             blob=blob,
-                                             use_pool_ptform=use_pool.get(
-                                                 'prior_transform', True))
+        live_points, logvol_init, init_ncalls = _initialize_live_points(
+            live_points,
+            ptform,
+            loglike,
+            M,
+            nlive=nlive,
+            npdim=npdim,
+            rstate=rstate,
+            blob=blob,
+            use_pool_ptform=use_pool.get('prior_transform', True))
 
         # Initialize our nested sampler.
         sampler = super().__new__(_SAMPLERS[bound])
@@ -665,11 +688,16 @@ optional
                          use_pool,
                          kwargs,
                          ncdim=ncdim,
-                         blob=blob)
-
+                         blob=blob,
+                         logvol_init=logvol_init)
+        sampler.ncalls = init_ncalls
         return sampler
 
 
+NestedSampler.__new__.__doc__ = _assemble_sampler_docstring(False)
+NestedSampler.__init__.__doc__ = _assemble_sampler_docstring(False)
+
+
 class DynamicNestedSampler(DynamicSampler):
     """
     The main class for performing dynamic nested sampling.
@@ -712,239 +740,6 @@ class DynamicNestedSampler(DynamicSampler):
                  blob=False,
                  save_history=False,
                  history_filename=None):
-        """
-        Initializes a sampler object for Dynamic Nested Sampling.
-
-        Parameters
-        ----------
-        loglikelihood : function
-            Function returning ln(likelihood) given parameters as a 1-d
-            `~numpy` array of length `ndim`.
-
-        prior_transform : function
-            Function translating a unit cube to the parameter space according
-            to the prior. The input is a 1-d `~numpy` array with length
-            `ndim`, where each value is in the range [0, 1). The return
-            value should also be a 1-d `~numpy` array with length `ndim`,
-            where each value is a parameter.
-            The return value is passed to the loglikelihood function. For
-            example, for a 2 parameter model with flat priors in the range
-            [0, 2), the
-            function would be::
-
-                def prior_transform(u):
-                    return 2.0 * u
-
-        ndim : int
-            Number of parameters returned by `prior_transform` and accepted by
-            `loglikelihood`.
-
-        bound : {`'none'`, `'single'`, `'multi'`, `'balls'`, `'cubes'`},
-            optional
-            Method used to approximately bound the prior using the current
-            set of live points. Conditions the sampling methods used to
-            propose new live points. Choices are no bound (`'none'`), a single
-            bounding ellipsoid (`'single'`), multiple bounding ellipsoids
-            (`'multi'`), balls centered on each live point (`'balls'`), and
-            cubes centered on each live point (`'cubes'`).
-            Default is `'multi'`.
-
-        sample : {`'auto'`, `'unif'`, `'rwalk'`, `'slice'`, `'rslice'`,
-            `'hslice'`}, optional
-            Method used to sample uniformly within the likelihood constraint,
-            conditioned on the provided bounds. Unique methods available are:
-            uniform sampling within the bounds(`'unif'`),
-            random walks with fixed proposals (`'rwalk'`),
-            multivariate slice sampling along preferred orientations
-            (`'slice'`),
-            "random" slice sampling along all orientations (`'rslice'`),
-            "Hamiltonian" slices along random trajectories (`'hslice'`), and
-            any callable function which follows the pattern of the sample
-            methods defined in dynesty.sampling.
-            `'auto'` selects the sampling method based on the dimensionality
-            of the problem (from `ndim`).
-            When `ndim < 10`, this defaults to `'unif'`.
-            When `10 <= ndim <= 20`, this defaults to `'rwalk'`.
-            When `ndim > 20`, this defaults to `'hslice'` if a `gradient` is
-            provided and `'rslice'` otherwise. `'slice'`
-            is provided as alternative for `'rslice'`.
-            Default is `'auto'`.
-        
-        periodic : iterable, optional
-            A list of indices for parameters with periodic boundary conditions.
-            These parameters *will not* have their positions constrained to be
-            within the unit cube, enabling smooth behavior for parameters
-            that may wrap around the edge. Default is `None` (i.e. no periodic
-            boundary conditions).
-
-        reflective : iterable, optional
-            A list of indices for parameters with reflective boundary
-            conditions.
-            These parameters *will not* have their positions constrained to be
-            within the unit cube, enabling smooth behavior for parameters
-            that may reflect at the edge. Default is `None` (i.e. no reflective
-            boundary conditions).
-
-        update_interval : int or float, optional
-            If an integer is passed, only update the proposal distribution
-            every `update_interval`-th likelihood call. If a float is passed,
-            update the
-            proposal after every `round(update_interval * nlive)`-th likelihood
-            call. Larger update intervals larger can be more efficient
-            when the likelihood function is quick to evaluate. Default behavior
-            is to target a roughly constant change in prior volume, with
-            `1.5` for `'unif'`, `0.15 * walks` for `'rwalk'`.
-            `0.9 * ndim * slices` for `'slice'`, `2.0 * slices` for `'rslice'`,
-            and `25.0 * slices` for `'hslice'`.
-
-        first_update : dict, optional
-            A dictionary containing parameters governing when the sampler
-            should first update the bounding distribution from the unit cube
-            (`'none'`)
-            to the one specified by `sample`. Options are the minimum number of
-            likelihood calls (`'min_ncall'`) and the minimum allowed overall
-            efficiency in percent (`'min_eff'`). Defaults are `2 * nlive` and
-            `10.`, respectively.
-
-        npdim : int, optional
-            Number of parameters accepted by `prior_transform`. This might
-            differ from `ndim` in the case where a parameter of loglikelihood
-            is dependent upon multiple independently distributed parameters,
-            some of which may
-            be nuisance parameters.
-
-        rstate : `~numpy.random.Generator`, optional
-            `~numpy.random.Generator` instance. If not given, the
-             global random state of the `~numpy.random` module will be used.
-
-        queue_size : int, optional
-            Carry out likelihood evaluations in parallel by queueing up new
-            live point proposals using (at most) `queue_size` many threads.
-            Each thread independently proposes new live points until the
-            proposal distribution
-            is updated. If no value is passed, this defaults to `pool.size`
-            (if a `pool` has been provided) and `1` otherwise (no parallelism).
-
-        pool : user-provided pool, optional
-            Use this pool of workers to execute operations in parallel.
-
-        use_pool : dict, optional
-            A dictionary containing flags indicating where a pool should be
-            used to execute operations in parallel. These govern whether
-            `prior_transform` is executed in parallel during initialization
-            (`'prior_transform'`),
-            `loglikelihood` is executed in parallel during initialization
-            (`'loglikelihood'`), live points are proposed in parallel during
-            a run
-            (`'propose_point'`), bounding distributions are updated in
-            parallel during a run (`'update_bound'`), and the stopping criteria
-            is evaluated in parallel during a run (`'stop_function'`).
-            Default is `True` for all options.
-
-        logl_args : iterable, optional
-            Additional arguments that can be passed to `loglikelihood`.
-
-        logl_kwargs : dict, optional
-            Additional keyword arguments that can be passed to `loglikelihood`.
-
-        ptform_args : iterable, optional
-            Additional arguments that can be passed to `prior_transform`.
-
-        ptform_kwargs : dict, optional
-            Additional keyword arguments that can be passed to
-            `prior_transform`.
-
-        gradient : function, optional
-            A function which returns the gradient corresponding to
-            the provided `loglikelihood` *with respect to the unit cube*.
-            If provided, this will be used when computing reflections
-            when sampling with `'hslice'`. If not provided, gradients are
-            approximated numerically using 2-sided differencing.
-
-        grad_args : iterable, optional
-            Additional arguments that can be passed to `gradient`.
-
-        grad_kwargs : dict, optional
-            Additional keyword arguments that can be passed to `gradient`.
-
-        compute_jac : bool, optional
-            Whether to compute and apply the Jacobian `dv/du`
-            from the target space `v` to the unit cube `u` when evaluating the
-            `gradient`. If `False`, the gradient provided is assumed to be
-            already defined with respect to the unit cube. If `True`, the
-            gradient provided is assumed to be defined with respect to the
-            target space so the Jacobian needs to be numerically computed
-            and applied. Default is `False`.
-
-        enlarge : float, optional
-            Enlarge the volumes of the specified bounding object(s) by this
-            fraction. The preferred method is to determine this organically
-            using bootstrapping. If `bootstrap > 0`, this defaults to `1.0`.
-            If `bootstrap = 0`, this instead defaults to `1.25`.
-
-        bootstrap : int, optional
-            Compute this many bootstrapped realizations of the bounding
-            objects. Use the maximum distance found to the set of points left
-            out during each iteration to enlarge the resulting volumes. Can
-            lead to unstable bounding ellipsoids. Default is `None` (no
-            bootstrap unless
-            the sampler is uniform). If bootstrap=0 then bootstrap is disabled.
-
-        walks : int, optional
-            For the `'rwalk'` sampling option, the minimum number of steps
-            (minimum 2) before proposing a new live point. Default is `25`.
-
-        facc : float, optional
-            The target acceptance fraction for the `'rwalk'` sampling option.
-            Default is `0.5`. Bounded to be between `[1. / walks, 1.]`.
-
-        slices : int, optional
-            For the `'slice'`, `'rslice'`, and `'hslice'` sampling
-            options, the number of times to execute a "slice update"
-            before proposing a new live point. Default is 3 for
-            `'slice'` and 3+ndim for rslice and hslice.
-            Note that `'slice'` cycles through **all dimensions**
-            when executing a "slice update".
-
-        fmove : float, optional
-            The target fraction of samples that are proposed along a trajectory
-            (i.e. not reflecting) for the `'hslice'` sampling option.
-            Default is `0.9`.
-
-        max_move : int, optional
-            The maximum number of timesteps allowed for `'hslice'`
-            per proposal forwards and backwards in time. Default is `100`.
-
-        update_func : function, optional
-            Any callable function which takes in a `blob` and `scale`
-            as input and returns a modification to the internal `scale`
-            as output.
-            Must follow the pattern of the update methods defined
-            in dynesty.nestedsamplers. If provided, this will supersede the
-            default functions used to update proposals. In the case where a
-            custom
-            callable function is passed to `sample` but no similar function is
-            passed to `update_func`, this will default to no update.
-
-        ncdim: int, optional
-            The number of clustering dimensions. The first ncdim dimensions
-            will
-            be sampled using the sampling method, the remaining dimensions will
-            just sample uniformly from the prior distribution.
-            If this is `None` (default), this will default to npdim.
-
-        blob: bool, optional 
-            The default value is False. If it is true, then the log-likelihood
-            should return the tuple of logl and a numpy-array "blob" that will 
-            stored as part of the chain. That blob can contain auxiliary 
-            information computed inside the likelihood function.
-
-        Returns
-        -------
-        sampler : a :class:`dynesty.DynamicSampler` instance
-            An initialized instance of the dynamic nested sampler.
-
-        """
 
         # Prior dimensions.
         if npdim is None:
@@ -1073,6 +868,9 @@ class DynamicNestedSampler(DynamicSampler):
                          queue_size, pool, use_pool, ncdim, nlive, kwargs)
 
 
+DynamicNestedSampler.__init__.__doc__ = _assemble_sampler_docstring(True)
+
+
 class _function_wrapper:
     """
     A hack to make functions pickleable when `args` or `kwargs` are
